package com.link.core.converter;

import com.link.core.converter.i18n.DefaultStringValue;
import com.link.core.converter.i18n.Resources;
import com.link.core.converter.util.RegularCloneable;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class ArrayListConverter<S, T> implements Converter<Object, ArrayList<T>>, HasConverters, RegularCloneable {
   private static final boolean DEFAULT_OPTIMAL = true;
   private Converter<S, T> elementConverter;
   private boolean optimal;

   public static <E> ArrayListConverter<E, E> newConverter(Class<E> elementType) {
      return new ArrayListConverter();
   }

   public static <S, T> ArrayListConverter<S, T> newConverter(Class<S> sourceElementType, Class<T> targetElementType) {
      return new ArrayListConverter();
   }

   public static <S, T> ArrayListConverter<S, T> newConverter(Converter<S, T> elementConverter) {
      return new ArrayListConverter(elementConverter);
   }

   protected ArrayListConverter() {
      this((Converter)null);
   }

   protected ArrayListConverter(Converter<S, T> elementConverter) {
      this.optimal = true;
      this.elementConverter = elementConverter;
   }

   public Converter<S, T> getElementConverter() {
      return this.elementConverter;
   }

   public ArrayListConverter<S, T> setElementConverter(Converter<S, T> elementConverter) {
      this.elementConverter = elementConverter;
      return this;
   }

   public boolean isOptimal() {
      return this.optimal;
   }

   public ArrayListConverter<S, T> setOptimal(boolean optimal) {
      this.optimal = optimal;
      return this;
   }

   public List<Converter> getConverters() {
      List<Converter> list = new ArrayList();
      if (this.elementConverter != null) {
         list.add(this.elementConverter);
      }

      return list;
   }

   public ArrayList<T> convert(Object source) throws ConversionException {
      if (source == null) {
         return null;
      } else {
         try {
            if (source instanceof Object[]) {
               return this.convertFromArray((Object[])((Object[])source));
            } else if (source instanceof Iterable) {
               return this.convertFromIterable((Iterable)source);
            } else {
               throw new IllegalArgumentException(MessageFormat.format(ArrayListConverter.R.R.unsupportedSourceClass(), source.getClass().getName()));
            }
         } catch (ClassCastException var3) {
            ClassCastException e = var3;
            throw new IllegalArgumentException(MessageFormat.format(ArrayListConverter.R.R.unsupportedSourceClass(), source.getClass().getName()), e);
         }
      }
   }

   public Object clone() {
      ArrayListConverter<S, T> other = new ArrayListConverter();
      other.copyFrom(this);
      return other;
   }

   public void copyFrom(Object source) {
      if (source != null && source instanceof ArrayListConverter) {
         ArrayListConverter<S, T> sc = (ArrayListConverter)source;
         if (sc.elementConverter instanceof RegularCloneable) {
            this.elementConverter = (Converter)((RegularCloneable)sc.elementConverter).clone();
         } else {
            this.elementConverter = sc.elementConverter;
         }

         this.optimal = sc.optimal;
      }
   }

   private ArrayList<T> convertFromArray(S[] source) throws ConversionException {
      assert source != null;

      ArrayList<T> target = new ArrayList();
      Map<S, T> buffer = new HashMap();
      Object[] var4 = source;
      int var5 = source.length;

      for(int var6 = 0; var6 < var5; ++var6) {
         S sourceElement = var4[var6];
         T targetElement = this.convertElement(sourceElement, buffer);
         target.add(targetElement);
      }

      return target;
   }

   private ArrayList<T> convertFromIterable(Iterable<S> source) throws ConversionException {
      assert source != null;

      ArrayList<T> target = new ArrayList();
      Map<S, T> buffer = new HashMap();
      Iterator var4 = source.iterator();

      while(var4.hasNext()) {
         S sourceElement = var4.next();
         T targetElement = this.convertElement(sourceElement, buffer);
         target.add(targetElement);
      }

      return target;
   }

   private T convertElement(S source, Map<S, T> buffer) throws ConversionException {
      if (this.elementConverter == null) {
         return source;
      } else if (!this.optimal) {
         return this.elementConverter.convert(source);
      } else {
         assert buffer != null;

         if (buffer.containsKey(source)) {
            return buffer.get(source);
         } else {
            T target = this.elementConverter.convert(source);
            buffer.put(source, target);
            return target;
         }
      }
   }

   public interface R {
      R R = (R)Resources.create(R.class);

      @DefaultStringValue("不支持source对象类型“{0}”。")
      String unsupportedSourceClass();
   }
}
